/**
 * Supports a jwt mechanism for addon user sessions. These sessions are typically
 * established off the back of incoming host requests and allow for subsequent
 * browser to addon host authentication.
 */
library ac_dart.ac_jwt.addon_session_token;

import 'package:dart_jwt/dart_jwt.dart';
import 'utils.dart';
import 'package:shelf_auth/shelf_auth.dart' show SessionClaimSet;
import 'package:logging/logging.dart';

Logger _log = new Logger('ac_dart.ac_jwt.addon_session_token');

/**
 * Creates a Jwt token suitable for inclusion in communications between the
 * addons client and server components
 */
@deprecated
String createAddonSessionToken(
    String addonSecret, String addonKey, String productHostKey, String user) {
  // TODO: need to configure how long it is valid for
  final claimSet = new AddonSessionClaimSet(addonKey, user, productHostKey);

  _log.finest('created claimSet: \n${claimSet.toJson()}');
  final jwt = new JsonWebToken.jws(
      claimSet, new JwaSymmetricKeySignatureContext(addonSecret));
  return jwt.encode();
}

/**
 * Decodes a Jwt token received from the addon client
 */
@deprecated
JsonWebToken<AddonSessionClaimSet> decodeAddonSessionToken(String jwtToken,
    {JwsValidationContext validationContext}) {
  return new JsonWebToken.decode(jwtToken,
      validationContext: validationContext,
      claimSetParser: (Map json) => new AddonSessionClaimSet.fromJson(json));
}

/**
 * A [JwtClaimSet] used for addon sessions. It includes a custom claim
 * [productHostKey] for identifying the product host that established the 
 * session originally for the user.
 * 
 * The user is included in the standard `sub` claim. The `issuer` is the addons key
 */
class AddonSessionClaimSet extends SessionClaimSet {
  final String productHostKey;

  AddonSessionClaimSet(String issuer, String subject, this.productHostKey,
      {String sessionIdentifier,
      Duration idleTimeout,
      Duration totalSessionTimeout})
      : super.create(issuer, subject,
            idleTimeout: idleTimeout, totalSessionTimeout: totalSessionTimeout);

  AddonSessionClaimSet.build(
      {String issuer,
      String subject,
      this.productHostKey,
      DateTime expiry,
      DateTime issuedAt,
      String audience,
      DateTime totalSessionExpiry,
      String sessionIdentifier})
      : super(issuer, subject, expiry, issuedAt, audience, sessionIdentifier,
            totalSessionExpiry);

  AddonSessionClaimSet.fromJson(Map json)
      : productHostKey = json['productHostKey'],
        super.fromJson(json);

  @override
  Map toJson() {
    return removeNullValues(
        super.toJson()..['productHostKey'] = productHostKey);
  }
}

/*
UserLookupBySessionClaimSet<P, CS> userLookup, this.sessionClaimFactory,

typedef CS SessionClaimFactory<CS extends SessionClaimSet>(String issuer,
    String subject, String sessionIdentifier, Duration idleTimeout,
    Duration totalSessionTimeout);

 */

final JwtCodec<AddonSessionClaimSet> addonSessionCodec =
    new JwtCodec.def((Map json,
            // TODO: validationContext not used. Is that OK?
            {JwsValidationContext validationContext}) =>
        new AddonSessionClaimSet.fromJson(json));
